@@ -1,9 +1,9 @@
-'''
+"""
 This module defines :class:`Event`, an array of events.
 
 :class:`Event` derives from :class:`BaseNeo`, from
 :module:`neo.core.baseneo`.
-'''
+"""
 
 from copy import deepcopy, copy
 
@@ -15,63 +15,96 @@ from neo.core.dataobject import DataObject, ArrayDict
 from neo.core.epoch import Epoch
 
 
-def _new_event(cls, times=None, labels=None, units=None, name=None, file_origin=None,
-               description=None, array_annotations=None, annotations=None, segment=None):
-    '''
+def _new_event(
+    cls,
+    times=None,
+    labels=None,
+    units=None,
+    name=None,
+    file_origin=None,
+    description=None,
+    array_annotations=None,
+    annotations=None,
+    segment=None,
+):
+    """
     A function to map Event.__new__ to function that
     does not do the unit checking. This is needed for pickle to work.
-    '''
-    e = Event(times=times, labels=labels, units=units, name=name, file_origin=file_origin,
-              description=description, array_annotations=array_annotations, **annotations)
+    """
+    e = Event(
+        times=times,
+        labels=labels,
+        units=units,
+        name=name,
+        file_origin=file_origin,
+        description=description,
+        array_annotations=array_annotations,
+        **annotations,
+    )
     e.segment = segment
     return e
 
 
 class Event(DataObject):
-    '''
-    Array of events.
+    """
+    Array of events which are the start times of events along with the labels
+    of the events
 
-    *Usage*::
+    Parameters
+    ----------
+    times: quantity array 1d | list
+        The times of the events
+    labels: numpy.ndarray 1d dtype='U' | list
+        Names or labels for the events
+    units: quantity units | None, default: None
+        If times are list the units of the times
+        If times is a quantity array this is ignored
+    name: str | None, default: None
+        An optional label for the dataset
+    description: str | None, default: None
+        An optional text descriptoin of the dataset
+    file_orgin: str | None, default: None
+        The filesystem path or url of the original data file
+    array_annotations: dict | None, default: None
+        Dict mapping strings to numpy arrays containing annotations for all data points
+    **annotations: dict
+        Additional user specified metadata stored in the annotations attribue
 
-        >>> from neo.core import Event
-        >>> from quantities import s
-        >>> import numpy as np
-        >>>
-        >>> evt = Event(np.arange(0, 30, 10)*s,
-        ...             labels=np.array(['trig0', 'trig1', 'trig2'],
-        ...                             dtype='U'))
-        >>>
-        >>> evt.times
-        array([  0.,  10.,  20.]) * s
-        >>> evt.labels
-        array(['trig0', 'trig1', 'trig2'],
+    Examples
+    --------
+
+    >>> from neo.core import Event
+    >>> from quantities import s
+    >>> import numpy as np
+    >>>
+    >>> evt = Event(np.arange(0, 30, 10)*s,
+    ...             labels=np.array(['trig0', 'trig1', 'trig2'],
+    ...                             dtype='U'))
+    >>>
+    >>> evt.times
+    array([  0.,  10.,  20.]) * s
+    >>> evt.labels
+    array(['trig0', 'trig1', 'trig2'],
               dtype='<U5')
 
-    *Required attributes/properties*:
-        :times: (quantity array 1D) The time of the events.
-        :labels: (numpy.array 1D dtype='U' or list) Names or labels for the events.
+    """
 
-    *Recommended attributes/properties*:
-        :name: (str) A label for the dataset.
-        :description: (str) Text description.
-        :file_origin: (str) Filesystem path or URL of the original data file.
+    _parent_objects = ("Segment",)
+    _parent_attrs = ("segment",)
+    _quantity_attr = "times"
+    _necessary_attrs = (("times", pq.Quantity, 1), ("labels", np.ndarray, 1, np.dtype("U")))
 
-    *Optional attributes/properties*:
-        :array_annotations: (dict) Dict mapping strings to numpy arrays containing annotations \
-                                   for all data points
-
-    Note: Any other additional arguments are assumed to be user-specific
-    metadata and stored in :attr:`annotations`.
-
-    '''
-
-    _parent_objects = ('Segment',)
-    _parent_attrs = ('segment',)
-    _quantity_attr = 'times'
-    _necessary_attrs = (('times', pq.Quantity, 1), ('labels', np.ndarray, 1, np.dtype('U')))
-
-    def __new__(cls, times=None, labels=None, units=None, name=None, description=None,
-                file_origin=None, array_annotations=None, **annotations):
+    def __new__(
+        cls,
+        times=None,
+        labels=None,
+        units=None,
+        name=None,
+        description=None,
+        file_origin=None,
+        array_annotations=None,
+        **annotations,
+    ):
         if times is None:
             times = np.array([]) * pq.s
         elif isinstance(times, (list, tuple)):
@@ -79,7 +112,7 @@ class Event(DataObject):
         if len(times.shape) > 1:
             raise ValueError("Times array has more than 1 dimension")
         if labels is None:
-            labels = np.array([], dtype='U')
+            labels = np.array([], dtype="U")
         else:
             labels = np.array(labels)
             if labels.size != times.size and labels.size:
@@ -90,90 +123,127 @@ class Event(DataObject):
                 units = times.units
                 dim = units.dimensionality
             except AttributeError:
-                raise ValueError('you must specify units')
+                raise ValueError("you must specify units")
         else:
-            if hasattr(units, 'dimensionality'):
+            if hasattr(units, "dimensionality"):
                 dim = units.dimensionality
             else:
                 dim = pq.quantity.validate_dimensionality(units)
         # check to make sure the units are time
         # this approach is much faster than comparing the
         # reference dimensionality
-        if (len(dim) != 1 or list(dim.values())[0] != 1 or not isinstance(list(dim.keys())[0],
-                                                                          pq.UnitTime)):
-            ValueError("Unit {} has dimensions {}, not [time]".format(units, dim.simplified))
+        if len(dim) != 1 or list(dim.values())[0] != 1 or not isinstance(list(dim.keys())[0], pq.UnitTime):
+            ValueError(f"Unit {units} has dimensions {dim.simplified}, not [time].")
 
         obj = pq.Quantity(times, units=dim).view(cls)
         obj._labels = labels
         obj.segment = None
         return obj
 
-    def __init__(self, times=None, labels=None, units=None, name=None, description=None,
-                 file_origin=None, array_annotations=None, **annotations):
-        '''
+    def __init__(
+        self,
+        times=None,
+        labels=None,
+        units=None,
+        name=None,
+        description=None,
+        file_origin=None,
+        array_annotations=None,
+        **annotations,
+    ):
+        """
         Initialize a new :class:`Event` instance.
-        '''
-        DataObject.__init__(self, name=name, file_origin=file_origin, description=description,
-                            array_annotations=array_annotations, **annotations)
+        """
+        DataObject.__init__(
+            self,
+            name=name,
+            file_origin=file_origin,
+            description=description,
+            array_annotations=array_annotations,
+            **annotations,
+        )
 
     def __reduce__(self):
-        '''
+        """
         Map the __new__ function onto _new_event, so that pickle
         works
-        '''
-        return _new_event, (self.__class__, np.array(self), self.labels, self.units, self.name,
-                            self.file_origin, self.description, self.array_annotations,
-                            self.annotations, self.segment)
+        """
+        return _new_event, (
+            self.__class__,
+            np.array(self),
+            self.labels,
+            self.units,
+            self.name,
+            self.file_origin,
+            self.description,
+            self.array_annotations,
+            self.annotations,
+            self.segment,
+        )
 
     def __array_finalize__(self, obj):
         super().__array_finalize__(obj)
-        self._labels = getattr(obj, 'labels', None)
-        self.annotations = getattr(obj, 'annotations', None)
-        self.name = getattr(obj, 'name', None)
-        self.file_origin = getattr(obj, 'file_origin', None)
-        self.description = getattr(obj, 'description', None)
-        self.segment = getattr(obj, 'segment', None)
+        self._labels = getattr(obj, "labels", None)
+        self.annotations = getattr(obj, "annotations", None)
+        self.name = getattr(obj, "name", None)
+        self.file_origin = getattr(obj, "file_origin", None)
+        self.description = getattr(obj, "description", None)
+        self.segment = getattr(obj, "segment", None)
         # Add empty array annotations, because they cannot always be copied,
         # but do not overwrite existing ones from slicing etc.
         # This ensures the attribute exists
-        if not hasattr(self, 'array_annotations'):
+        if not hasattr(self, "array_annotations"):
             self.array_annotations = ArrayDict(self._get_arr_ann_length())
 
     def __repr__(self):
-        '''
+        """
         Returns a string representing the :class:`Event`.
-        '''
+        """
 
-        objs = ['%s@%s' % (label, str(time)) for label, time in zip(self.labels, self.times)]
-        return '<Event: %s>' % ', '.join(objs)
+        objs = [f"{label}@{str(time)}" for label, time in zip(self.labels, self.times)]
+        return "<Event: %s>" % ", ".join(objs)
 
     def _repr_pretty_(self, pp, cycle):
         labels = ""
         if self._labels is not None:
             labels = " with labels"
-        pp.text(f"{self.__class__.__name__} containing {self.size} events{labels}; "
-        f"time units {self.units.dimensionality.string}; datatype {self.dtype} ")
+        pp.text(
+            f"{self.__class__.__name__} containing {self.size} events{labels}; "
+            f"time units {self.units.dimensionality.string}; datatype {self.dtype} "
+        )
         if self._has_repr_pretty_attrs_():
             pp.breakable()
             self._repr_pretty_attrs_(pp, cycle)
 
     def rescale(self, units, dtype=None):
-        '''
+        """
         Return a copy of the :class:`Event` converted to the specified units
+
+        Parameters
+        ----------
+        units: quantity units
+            The units to convert the Event to
+        dtype: None
+            Exists for backward compatiblity within quantities see Notes for more info
+
+        Returns
+        -------
+        obj: neo.core.Event
+            A copy of the event with the specified units
+
+        Notes
+        -----
         The `dtype` argument exists only for backward compatibility within quantities, see
         https://github.com/python-quantities/python-quantities/pull/204
-        :return: Copy of self with specified units
-        '''
+
+        """
         # Use simpler functionality, if nothing will be changed
         dim = pq.quantity.validate_dimensionality(units)
         if self.dimensionality == dim:
             return self.copy()
 
         # Rescale the object into a new object
-        obj = self.duplicate_with_new_data(
-            times=self.view(pq.Quantity).rescale(dim),
-            labels=self.labels,
-            units=units)
+        obj = self.duplicate_with_new_data(times=self.view(pq.Quantity).rescale(dim), labels=self.labels, units=units)
 
         # Expected behavior is deepcopy, so deepcopying array_annotations
         obj.array_annotations = deepcopy(self.array_annotations)
@@ -185,15 +255,22 @@ class Event(DataObject):
         return pq.Quantity(self)
 
     def merge(self, other):
-        '''
-        Merge the another :class:`Event` into this one.
+        """
+        Merge another :class:`Event` into this one.
 
-        The :class:`Event` objects are concatenated horizontally
+        Parameter
+        ---------
+        other: neo.core.Event
+            The `Event` to merge into this one
+
+        Notes
+        -----
+        * The :class:`Event` objects are concatenated horizontally
         (column-wise), :func:`np.hstack`).
 
-        If the attributes of the two :class:`Event` are not
+        * If the attributes of the two :class:`Event` are not
         compatible, and Exception is raised.
-        '''
+        """
         othertimes = other.times.rescale(self.times.units)
         times = np.hstack([self.times, othertimes]) * self.times.units
         labels = np.hstack([self.labels, other.labels])
@@ -204,29 +281,28 @@ class Event(DataObject):
             if attr_self == attr_other:
                 kwargs[name] = attr_self
             else:
-                kwargs[name] = "merge({}, {})".format(attr_self, attr_other)
+                kwargs[name] = f"merge({attr_self}, {attr_other})"
 
-        print('Event: merge annotations')
+        print("Event: merge annotations")
         merged_annotations = merge_annotations(self.annotations, other.annotations)
 
         kwargs.update(merged_annotations)
 
-        kwargs['array_annotations'] = self._merge_array_annotations(other)
+        kwargs["array_annotations"] = self._merge_array_annotations(other)
 
         evt = Event(times=times, labels=labels, **kwargs)
 
         return evt
 
     def _copy_data_complement(self, other):
-        '''
+        """
         Copy the metadata from another :class:`Event`.
         Note: Array annotations can not be copied here because length of data can change
-        '''
+        """
         # Note: Array annotations, including labels, cannot be copied
         # because they are linked to their respective timestamps and length of data can be changed
         # here which would cause inconsistencies
-        for attr in ("name", "file_origin", "description",
-                     "annotations"):
+        for attr in ("name", "file_origin", "description", "annotations"):
             setattr(self, attr, deepcopy(getattr(other, attr, None)))
 
     def __getitem__(self, i):
@@ -244,8 +320,7 @@ class Event(DataObject):
 
     def set_labels(self, labels):
         if self.labels is not None and self.labels.size > 0 and len(labels) != self.size:
-            raise ValueError("Labels array has different length to times ({} != {})"
-                            .format(len(labels), self.size))
+            raise ValueError(f"Labels array has different length to times " f"({len(labels)} != {self.size})")
         self._labels = np.array(labels)
 
     def get_labels(self):
@@ -254,11 +329,11 @@ class Event(DataObject):
     labels = property(get_labels, set_labels)
 
     def duplicate_with_new_data(self, times, labels, units=None):
-        '''
+        """
         Create a new :class:`Event` with the same metadata
         but different data
         Note: Array annotations can not be copied here because length of data can change
-        '''
+        """
         if units is None:
             units = self.units
         else:
@@ -271,12 +346,24 @@ class Event(DataObject):
         return new
 
     def time_slice(self, t_start, t_stop):
-        '''
-        Creates a new :class:`Event` corresponding to the time slice of
-        the original :class:`Event` between (and including) times
-        :attr:`t_start` and :attr:`t_stop`. Either parameter can also be None
-        to use infinite endpoints for the time interval.
-        '''
+        """
+        Creates a new `Event` corresponding to the time slice of the original `Event` between (and including) times
+        `t_start` and `t_stop`.
+
+        Parameters
+        ----------
+        t_start: float | None
+            The starting time of the time slice
+            If None will use -np.inf for the starting time
+        t_stop: float | None
+            The stopping time of the time slice
+            If None will use np.inf for the stopping time
+
+        Returns
+        -------
+        new_evt: neo.core.Event
+            The new `Event` limited by the time points
+        """
         _t_start = t_start
         _t_stop = t_stop
         if t_start is None:
@@ -306,8 +393,7 @@ class Event(DataObject):
             New instance of an :class:`Event` object starting at t_shift later than the
             original :class:`Event` (the original :class:`Event` is not modified).
         """
-        new_evt = self.duplicate_with_new_data(times=self.times + t_shift,
-                                               labels=self.labels)
+        new_evt = self.duplicate_with_new_data(times=self.times + t_shift, labels=self.labels)
 
         # Here we can safely copy the array annotations since we know that
         # the length of the Event does not change.
@@ -345,21 +431,17 @@ class Event(DataObject):
         if pairwise:
             # Mode 2
             if durations is not None:
-                raise ValueError("Inconsistent arguments. "
-                                 "Cannot give both `pairwise` and `durations`")
+                raise ValueError("Inconsistent arguments. " "Cannot give both `pairwise` and `durations`")
             if self.size % 2 != 0:
-                raise ValueError("Pairwise conversion of events to epochs"
-                                 " requires an even number of events")
+                raise ValueError("Pairwise conversion of events to epochs" " requires an even number of events")
             times = self.times[::2]
             durations = self.times[1::2] - times
-            labels = np.array(
-                ["{}-{}".format(a, b) for a, b in zip(self.labels[::2], self.labels[1::2])])
+            labels = np.array([f"{a}-{b}" for a, b in zip(self.labels[::2], self.labels[1::2])])
         elif durations is None:
             # Mode 1
             times = self.times[:-1]
             durations = np.diff(self.times)
-            labels = np.array(
-                ["{}-{}".format(a, b) for a, b in zip(self.labels[:-1], self.labels[1:])])
+            labels = np.array([f"{a}-{b}" for a, b in zip(self.labels[:-1], self.labels[1:])])
         else:
             # Mode 3
             times = self.times
