@@ -9,6 +9,7 @@ Supported: Read
 Author: Mikkel E. Lepperød @CINPLA
 
 """
+
 # TODO: writing to file
 
 import numpy as np
@@ -35,8 +36,7 @@ class KwikIO(BaseIO):
     is_readable = True  # This class can only read data
     is_writable = False  # write is not supported
 
-    supported_objects = [Block, Segment, SpikeTrain, AnalogSignal,
-                         Group]
+    supported_objects = [Block, Segment, SpikeTrain, AnalogSignal, Group]
 
     # This class can return either a Block or a Segment
     # The first one is the default ( self.read )
@@ -50,10 +50,10 @@ class KwikIO(BaseIO):
     has_header = False
     is_streameable = False
 
-    name = 'Kwik'
-    description = 'This IO reads experimental data from a .kwik dataset'
-    extensions = ['kwik']
-    mode = 'file'
+    name = "Kwik"
+    description = "This IO reads experimental data from a .kwik dataset"
+    extensions = ["kwik"]
+    mode = "file"
 
     def __init__(self, filename):
         """
@@ -65,16 +65,16 @@ class KwikIO(BaseIO):
         BaseIO.__init__(self)
         self.filename = os.path.abspath(filename)
         model = kwik.KwikModel(self.filename)  # TODO this group is loaded twice
-        self.models = [kwik.KwikModel(self.filename, channel_group=grp)
-                       for grp in model.channel_groups]
+        self.models = [kwik.KwikModel(self.filename, channel_group=grp) for grp in model.channel_groups]
 
-    def read_block(self,
-                   lazy=False,
-                   get_waveforms=True,
-                   cluster_group=None,
-                   raw_data_units='uV',
-                   get_raw_data=False,
-                   ):
+    def read_block(
+        self,
+        lazy=False,
+        get_waveforms=True,
+        cluster_group=None,
+        raw_data_units="uV",
+        get_raw_data=False,
+    ):
         """
         Reads a block with segments and groups
 
@@ -90,18 +90,16 @@ class KwikIO(BaseIO):
             Which clusters to load, possibilities are "noise", "unsorted",
             "good", if None all is loaded.
         """
-        assert not lazy, 'Do not support lazy'
+        assert not lazy, "Do not support lazy"
 
         blk = Block()
         seg = Segment(file_origin=self.filename)
         blk.segments += [seg]
         for model in self.models:
             group_id = model.channel_group
-            group_meta = {'group_id': group_id}
+            group_meta = {"group_id": group_id}
             group_meta.update(model.metadata)
-            chx = Group(name='channel group #{}'.format(group_id),
-                               index=model.channels,
-                               **group_meta)
+            chx = Group(name=f"channel group #{group_id}", index=model.channels, **group_meta)
             blk.groups.append(chx)
             clusters = model.spike_clusters
             for cluster_id in model.cluster_ids:
@@ -110,15 +108,11 @@ class KwikIO(BaseIO):
                     pass
                 elif cluster_group != meta:
                     continue
-                sptr = self.read_spiketrain(cluster_id=cluster_id,
-                                            model=model,
-                                            get_waveforms=get_waveforms,
-                                            raw_data_units=raw_data_units)
-                sptr.annotations.update({'cluster_group': meta,
-                                         'group_id': model.channel_group})
-                unit = Group(cluster_group=meta,
-                             group_id=model.channel_group,
-                             name='unit #{}'.format(cluster_id))
+                sptr = self.read_spiketrain(
+                    cluster_id=cluster_id, model=model, get_waveforms=get_waveforms, raw_data_units=raw_data_units
+                )
+                sptr.annotations.update({"cluster_group": meta, "group_id": model.channel_group})
+                unit = Group(cluster_group=meta, group_id=model.channel_group, name=f"unit #{cluster_id}")
                 unit.add(sptr)
                 chx.add(unit)
                 seg.spiketrains.append(sptr)
@@ -129,10 +123,10 @@ class KwikIO(BaseIO):
 
         seg.duration = model.duration * pq.s
 
-        blk.create_many_to_one_relationship()
+        blk.check_relationships()
         return blk
 
-    def read_analogsignal(self, model, units='uV', lazy=False):
+    def read_analogsignal(self, model, units="uV", lazy=False):
         """
         Reads analogsignals
 
@@ -141,19 +135,15 @@ class KwikIO(BaseIO):
         units: str, default = "uV"
             SI units of the raw trace according to voltage_gain given to klusta
         """
-        assert not lazy, 'Do not support lazy'
+        assert not lazy, "Do not support lazy"
 
-        arr = model.traces[:] * model.metadata['voltage_gain']
-        ana = AnalogSignal(arr, sampling_rate=model.sample_rate * pq.Hz,
-                           units=units,
-                           file_origin=model.metadata['raw_data_files'])
+        arr = model.traces[:] * model.metadata["voltage_gain"]
+        ana = AnalogSignal(
+            arr, sampling_rate=model.sample_rate * pq.Hz, units=units, file_origin=model.metadata["raw_data_files"]
+        )
         return ana
 
-    def read_spiketrain(self, cluster_id, model,
-                        lazy=False,
-                        get_waveforms=True,
-                        raw_data_units=None
-                        ):
+    def read_spiketrain(self, cluster_id, model, lazy=False, get_waveforms=True, raw_data_units=None):
         """
         Reads sorted spiketrains
 
@@ -167,10 +157,10 @@ class KwikIO(BaseIO):
             A KwikModel object obtained by klusta.kwik.KwikModel(fname)
         """
         try:
-            if (not (cluster_id in model.cluster_ids)):
+            if not (cluster_id in model.cluster_ids):
                 raise ValueError
         except ValueError:
-            print("Exception: cluster_id (%d) not found !! " % cluster_id)
+            print(f"Exception: cluster_id ({cluster_id}) not found !! ")
             return
         clusters = model.spike_clusters
         idx = np.nonzero(clusters == cluster_id)
@@ -181,13 +171,17 @@ class KwikIO(BaseIO):
             w = pq.Quantity(w, raw_data_units)
         else:
             w = None
-        if model.duration > 0.:
+        if model.duration > 0.0:
             t_stop = model.duration
         else:
             t_stop = np.max(model.spike_times[idx])
-        sptr = SpikeTrain(times=model.spike_times[idx],
-                          t_stop=t_stop, waveforms=w, units='s',
-                          sampling_rate=model.sample_rate * pq.Hz,
-                          file_origin=self.filename,
-                          **{'cluster_id': cluster_id})
+        sptr = SpikeTrain(
+            times=model.spike_times[idx],
+            t_stop=t_stop,
+            waveforms=w,
+            units="s",
+            sampling_rate=model.sample_rate * pq.Hz,
+            file_origin=self.filename,
+            **{"cluster_id": cluster_id},
+        )
         return sptr
