@@ -23,11 +23,19 @@ class NeuralynxIO(NeuralynxRawIO, BaseFromRaw):
     NSE contains spikes and waveforms for mono electrodes
     NTT contains spikes and waveforms for tetrodes
     """
-    _prefered_signal_group_mode = 'group-by-same-units'
-    mode = 'dir'
 
-    def __init__(self, dirname='', filename='', use_cache=False, cache_path='same_as_resource',
-                 exclude_filename=None, keep_original_times=False):
+    _prefered_signal_group_mode = "group-by-same-units"
+    mode = "dir"
+
+    def __init__(
+        self,
+        dirname="",
+        filename="",
+        use_cache=False,
+        cache_path="same_as_resource",
+        exclude_filename=None,
+        keep_original_times=False,
+    ):
         """
         Initialise IO instance
 
@@ -52,10 +60,16 @@ class NeuralynxIO(NeuralynxRawIO, BaseFromRaw):
             shifted to begin at t_start = 0*pq.second.
             Default: False
         """
-        NeuralynxRawIO.__init__(self, dirname=dirname, filename=filename, use_cache=use_cache,
-                                cache_path=cache_path, exclude_filename=exclude_filename,
-                                keep_original_times=keep_original_times)
-        if self.rawmode == 'one-file':
+        NeuralynxRawIO.__init__(
+            self,
+            dirname=dirname,
+            filename=filename,
+            use_cache=use_cache,
+            cache_path=cache_path,
+            exclude_filename=exclude_filename,
+            keep_original_times=keep_original_times,
+        )
+        if self.rawmode == "one-file":
             BaseFromRaw.__init__(self, filename)
-        elif self.rawmode == 'one-dir':
+        elif self.rawmode == "one-dir":
             BaseFromRaw.__init__(self, dirname)
