@@ -16,7 +16,7 @@ import numpy as np
 import quantities as pq
 
 from neo.io.baseio import BaseIO
-from neo.core import Segment, AnalogSignal
+from neo.core import Segment, AnalogSignal, NeoReadWriteError
 
 from neo.io.basefromrawio import BaseFromRaw
 from neo.rawio.rawbinarysignalrawio import RawBinarySignalRawIO
@@ -40,7 +40,7 @@ class RawBinarySignalIO(RawBinarySignalRawIO, BaseFromRaw):
 
     """
 
-    _prefered_signal_group_mode = 'group-by-same-units'
+    _prefered_signal_group_mode = "group-by-same-units"
 
     is_readable = True
     is_writable = True
@@ -49,12 +49,26 @@ class RawBinarySignalIO(RawBinarySignalRawIO, BaseFromRaw):
     readable_objects = [Segment]
     writeable_objects = [Segment]
 
-    def __init__(self, filename, dtype='int16', sampling_rate=10000.,
-                 nb_channel=2, signal_gain=1., signal_offset=0., bytesoffset=0):
-        RawBinarySignalRawIO.__init__(self, filename=filename, dtype=dtype,
-                                      sampling_rate=sampling_rate, nb_channel=nb_channel,
-                                      signal_gain=signal_gain,
-                                      signal_offset=signal_offset, bytesoffset=bytesoffset)
+    def __init__(
+        self,
+        filename,
+        dtype="int16",
+        sampling_rate=10000.0,
+        nb_channel=2,
+        signal_gain=1.0,
+        signal_offset=0.0,
+        bytesoffset=0,
+    ):
+        RawBinarySignalRawIO.__init__(
+            self,
+            filename=filename,
+            dtype=dtype,
+            sampling_rate=sampling_rate,
+            nb_channel=nb_channel,
+            signal_gain=signal_gain,
+            signal_offset=signal_offset,
+            bytesoffset=bytesoffset,
+        )
         BaseFromRaw.__init__(self, filename)
 
     def write_segment(self, segment):
@@ -71,23 +85,27 @@ class RawBinarySignalIO(RawBinarySignalRawIO, BaseFromRaw):
         """
 
         if self.bytesoffset:
-            raise NotImplementedError('bytesoffset values other than 0 ' +
-                                      'not supported')
+            raise NotImplementedError("bytesoffset values other than 0 " + "not supported")
 
         anasigs = segment.analogsignals
-        assert len(anasigs) > 0, 'No AnalogSignal'
+        if len(anasigs) == 0:
+            raise NeoReadWriteError("No AnalogSignal to write")
 
         anasig0 = anasigs[0]
         if len(anasigs) == 1 and anasig0.ndim == 2:
             numpy_sigs = anasig0.magnitude
         else:
 
-            assert anasig0.ndim == 1 or (anasig0.ndim == 2 and anasig0.shape[1] == 1)
+            if anasig0.ndim != 1 and (anasig0.ndim == 2 and anasig0.shape[1] != 1):
+                raise NeoReadWriteError("Incorrect analogsignal shaping for write")
             # all AnaologSignal from Segment must have the same length/sampling_rate/dtype
             for anasig in anasigs[1:]:
-                assert anasig.shape == anasig0.shape
-                assert anasig.sampling_rate == anasig0.sampling_rate
-                assert anasig.dtype == anasig0.dtype
+                if anasig.shape != anasig0.shape:
+                    raise ValueError("The shape of one of the analog signals does not match")
+                if anasig.sampling_rate != anasig0.sampling_rate:
+                    raise ValueError("The sampling_rate of one of the analog signals does not match")
+                if anasig.dtype != anasig0.dtype:
+                    raise ValueError("The dtype of one the analog signals does not match")
 
             numpy_sigs = np.empty((anasig0.size, len(anasigs)))
             for i, anasig in enumerate(anasigs):
@@ -97,5 +115,5 @@ class RawBinarySignalIO(RawBinarySignalRawIO, BaseFromRaw):
         numpy_sigs /= self.signal_gain
         numpy_sigs = numpy_sigs.astype(self.dtype)
 
-        with open(self.filename, 'wb') as f:
+        with open(self.filename, "wb") as f:
             f.write(numpy_sigs.tobytes())
