@@ -9,18 +9,24 @@ https://bitbucket.org/mkarlsso/trodes/wiki/Configuration
 
 Note :
   * this file format have multiple version. news version include the gain for scaling.
-     The actual implementation do not contain this feature because we don't have
-     files to test this. So now the gain is "hardcoded" to 1. and so units
-     is not handled correctly.
+     The actual implementation does not contain this feature because we don't have
+     files to test this. So now the gain is "hardcoded" to 1. and so units are
+      not handled correctly.
 
 The ".rec" file format contains:
-  * a first  text part with information in an XML structure
+  * a first text part with information in an XML structure
   * a second part for the binary buffer
 
 Author: Samuel Garcia
 """
-from .baserawio import (BaseRawIO, _signal_channel_dtype, _signal_stream_dtype,
-    _spike_channel_dtype, _event_channel_dtype)
+
+from .baserawio import (
+    BaseRawIO,
+    _signal_channel_dtype,
+    _signal_stream_dtype,
+    _spike_channel_dtype,
+    _event_channel_dtype,
+)
 
 import numpy as np
 
@@ -28,24 +34,43 @@ from xml.etree import ElementTree
 
 
 class SpikeGadgetsRawIO(BaseRawIO):
-    extensions = ['rec']
-    rawmode = 'one-file'
+    extensions = ["rec"]
+    rawmode = "one-file"
 
-    def __init__(self, filename='', selected_streams=None):
+    def __init__(self, filename="", selected_streams=None):
         """
         Class for reading spikegadgets files.
         Only continuous signals are supported at the moment.
 
         Initialize a SpikeGadgetsRawIO for a single ".rec" file.
 
-        Args:
-            filename: str
-                The filename
-            selected_streams: None, list, str
-                sublist of streams to load/expose to API
-                useful for spikeextractor when one stream only is needed.
-                For instance streams = ['ECU', 'trodes']
-                'trodes' is name for ephy channel (ntrodes)
+        Parameters
+        ----------
+        filename: str, default: ''
+            The *.rec file to be loaded
+        selected_streams: str | list | None, default: None
+            sublist of streams to load/expose to API, e.g., ['ECU', 'trodes']
+            'trodes' is name for ephy channel (ntrodes)
+            None will keep all streams
+
+        Notes
+        -----
+        This file format has multiple versions:
+            - Newer versions include the gain for scaling to microvolts [uV].
+            - If the scaling is not found in the header, the gain will be "hardcoded" to 1,
+              in which case the units are not handled correctly.
+        This will not affect functions that do not rely on the data having physical units,
+            e.g., _get_analogsignal_chunk, but functions such as rescale_signal_raw_to_float
+            will be inaccurate.
+
+        Examples
+        --------
+        >>> import neo.rawio
+        >>> reader = neo.rawio.SpikeGadgetRawIO(filename='data.rec') # all streams
+        # just the electrode channels
+        >>> reader_trodes = neo.rawio.SpikeGadgetRawIO(filename='data.rec', selected_streams='trodes')
+
+
         """
         BaseRawIO.__init__(self)
         self.filename = filename
@@ -57,7 +82,7 @@ class SpikeGadgetsRawIO(BaseRawIO):
     def _parse_header(self):
         # parse file until "</Configuration>"
         header_size = None
-        with open(self.filename, mode='rb') as f:
+        with open(self.filename, mode="rb") as f:
             while True:
                 line = f.readline()
                 if b"</Configuration>" in line:
@@ -68,26 +93,27 @@ class SpikeGadgetsRawIO(BaseRawIO):
                 ValueError("SpikeGadgets: the xml header does not contain '</Configuration>'")
 
             f.seek(0)
-            header_txt = f.read(header_size).decode('utf8')
+            header_txt = f.read(header_size).decode("utf8")
 
         # explore xml header
         root = ElementTree.fromstring(header_txt)
-        gconf = sr = root.find('GlobalConfiguration')
-        hconf = root.find('HardwareConfiguration')
-        sconf = root.find('SpikeConfiguration')
+        gconf = sr = root.find("GlobalConfiguration")
+        hconf = root.find("HardwareConfiguration")
+        sconf = root.find("SpikeConfiguration")
 
-        self._sampling_rate = float(hconf.attrib['samplingRate'])
-        num_ephy_channels = int(hconf .attrib['numChannels'])
+        self._sampling_rate = float(hconf.attrib["samplingRate"])
+        num_ephy_channels = int(hconf.attrib["numChannels"])
 
         # explore sub stream and count packet size
         # first bytes is 0x55
         packet_size = 1
         stream_bytes = {}
         for device in hconf:
-            stream_id = device.attrib['name']
-            num_bytes = int(device.attrib['numBytes'])
-            stream_bytes[stream_id] = packet_size
-            packet_size += num_bytes
+            stream_id = device.attrib["name"]
+            if "numBytes" in device.attrib.keys():
+                num_bytes = int(device.attrib["numBytes"])
+                stream_bytes[stream_id] = packet_size
+                packet_size += num_bytes
 
         # timestamps 4 uint32
         self._timestamp_byte = packet_size
@@ -96,10 +122,10 @@ class SpikeGadgetsRawIO(BaseRawIO):
         packet_size += 2 * num_ephy_channels
 
         # read the binary part lazily
-        raw_memmap = np.memmap(self.filename, mode='r', offset=header_size, dtype='<u1')
+        raw_memmap = np.memmap(self.filename, mode="r", offset=header_size, dtype="<u1")
 
         num_packet = raw_memmap.size // packet_size
-        raw_memmap = raw_memmap[:num_packet * packet_size]
+        raw_memmap = raw_memmap[: num_packet * packet_size]
         self._raw_memmap = raw_memmap.reshape(-1, packet_size)
 
         # create signal channels
@@ -110,14 +136,14 @@ class SpikeGadgetsRawIO(BaseRawIO):
         # walk in xml device and keep only "analog" one
         self._mask_channels_bytes = {}
         for device in hconf:
-            stream_id = device.attrib['name']
+            stream_id = device.attrib["name"]
             for channel in device:
 
-                if 'interleavedDataIDByte' in channel.attrib:
+                if "interleavedDataIDByte" in channel.attrib:
                     # TODO LATER: deal with "headstageSensor" which have interleaved
                     continue
 
-                if channel.attrib['dataType'] == 'analog':
+                if ("dataType" in channel.attrib.keys()) and (channel.attrib["dataType"] == "analog"):
 
                     if stream_id not in stream_ids:
                         stream_ids.append(stream_id)
@@ -125,41 +151,54 @@ class SpikeGadgetsRawIO(BaseRawIO):
                         signal_streams.append((stream_name, stream_id))
                         self._mask_channels_bytes[stream_id] = []
 
-                    name = channel.attrib['id']
-                    chan_id = channel.attrib['id']
-                    dtype = 'int16'
+                    name = channel.attrib["id"]
+                    chan_id = channel.attrib["id"]
+                    dtype = "int16"
                     # TODO LATER : handle gain correctly according the file version
-                    units = ''
-                    gain = 1.
-                    offset = 0.
-                    signal_channels.append((name, chan_id, self._sampling_rate, 'int16',
-                                         units, gain, offset, stream_id))
+                    units = ""
+                    gain = 1.0
+                    offset = 0.0
+                    signal_channels.append(
+                        (name, chan_id, self._sampling_rate, "int16", units, gain, offset, stream_id)
+                    )
 
-                    num_bytes = stream_bytes[stream_id] + int(channel.attrib['startByte'])
-                    chan_mask = np.zeros(packet_size, dtype='bool')
+                    num_bytes = stream_bytes[stream_id] + int(channel.attrib["startByte"])
+                    chan_mask = np.zeros(packet_size, dtype="bool")
                     chan_mask[num_bytes] = True
                     chan_mask[num_bytes + 1] = True
                     self._mask_channels_bytes[stream_id].append(chan_mask)
 
         if num_ephy_channels > 0:
-            stream_id = 'trodes'
+            stream_id = "trodes"
             stream_name = stream_id
             signal_streams.append((stream_name, stream_id))
             self._mask_channels_bytes[stream_id] = []
 
             chan_ind = 0
-            for trode in sconf:
-                for schan in trode:
-                    name = 'trode' + trode.attrib['id'] + 'chan' + schan.attrib['hwChan']
-                    chan_id = schan.attrib['hwChan']
-                    # TODO LATER : handle gain correctly according the file version
-                    units = ''
-                    gain = 1.
-                    offset = 0.
-                    signal_channels.append((name, chan_id, self._sampling_rate, 'int16',
-                                         units, gain, offset, stream_id))
+            self.is_scaleable = "spikeScalingToUv" in sconf[0].attrib
+            if not self.is_scaleable:
+                self.logger.warning(
+                    "Unable to read channel gain scaling (to uV) from .rec header. Data has no physical units!"
+                )
 
-                    chan_mask = np.zeros(packet_size, dtype='bool')
+            for trode in sconf:
+                if "spikeScalingToUv" in trode.attrib:
+                    gain = float(trode.attrib["spikeScalingToUv"])
+                    units = "uV"
+                else:
+                    gain = 1  # revert to hardcoded gain
+                    units = ""
+
+                for schan in trode:
+                    name = "trode" + trode.attrib["id"] + "chan" + schan.attrib["hwChan"]
+                    chan_id = schan.attrib["hwChan"]
+
+                    offset = 0.0
+                    signal_channels.append(
+                        (name, chan_id, self._sampling_rate, "int16", units, gain, offset, stream_id)
+                    )
+
+                    chan_mask = np.zeros(packet_size, dtype="bool")
                     num_bytes = packet_size - 2 * num_ephy_channels + 2 * chan_ind
                     chan_mask[num_bytes] = True
                     chan_mask[num_bytes + 1] = True
@@ -181,12 +220,15 @@ class SpikeGadgetsRawIO(BaseRawIO):
         if self.selected_streams is not None:
             if isinstance(self.selected_streams, str):
                 self.selected_streams = [self.selected_streams]
-            assert isinstance(self.selected_streams, list)
+            if not isinstance(self.selected_streams, list):
+                raise TypeError(
+                    f"`selected_streams` must be of type str or list not of type {type(self.selected_streams)}"
+                )
 
-            keep = np.in1d(signal_streams['id'], self.selected_streams)
+            keep = np.isin(signal_streams["id"], self.selected_streams)
             signal_streams = signal_streams[keep]
 
-            keep = np.in1d(signal_channels['stream_id'], self.selected_streams)
+            keep = np.isin(signal_channels["stream_id"], self.selected_streams)
             signal_channels = signal_channels[keep]
 
         # No events channels
@@ -199,22 +241,22 @@ class SpikeGadgetsRawIO(BaseRawIO):
 
         # fille into header dict
         self.header = {}
-        self.header['nb_block'] = 1
-        self.header['nb_segment'] = [1]
-        self.header['signal_streams'] = signal_streams
-        self.header['signal_channels'] = signal_channels
-        self.header['spike_channels'] = spike_channels
-        self.header['event_channels'] = event_channels
+        self.header["nb_block"] = 1
+        self.header["nb_segment"] = [1]
+        self.header["signal_streams"] = signal_streams
+        self.header["signal_channels"] = signal_channels
+        self.header["spike_channels"] = spike_channels
+        self.header["event_channels"] = event_channels
 
         self._generate_minimal_annotations()
         # info from GlobalConfiguration in xml are copied to block and seg annotations
-        bl_ann = self.raw_annotations['blocks'][0]
-        seg_ann = self.raw_annotations['blocks'][0]['segments'][0]
+        bl_ann = self.raw_annotations["blocks"][0]
+        seg_ann = self.raw_annotations["blocks"][0]["segments"][0]
         for ann in (bl_ann, seg_ann):
             ann.update(gconf.attrib)
 
     def _segment_t_start(self, block_index, seg_index):
-        return 0.
+        return 0.0
 
     def _segment_t_stop(self, block_index, seg_index):
         size = self._raw_memmap.shape[0]
@@ -226,11 +268,10 @@ class SpikeGadgetsRawIO(BaseRawIO):
         return size
 
     def _get_signal_t_start(self, block_index, seg_index, stream_index):
-        return 0.
+        return 0.0
 
-    def _get_analogsignal_chunk(self, block_index, seg_index, i_start, i_stop, stream_index,
-                                channel_indexes):
-        stream_id = self.header['signal_streams'][stream_index]['id']
+    def _get_analogsignal_chunk(self, block_index, seg_index, i_start, i_stop, stream_index, channel_indexes):
+        stream_id = self.header["signal_streams"][stream_index]["id"]
 
         raw_unit8 = self._raw_memmap[i_start:i_stop]
 
@@ -249,10 +290,9 @@ class SpikeGadgetsRawIO(BaseRawIO):
                 if np.any(np.diff(channel_indexes) < 0):
                     # handle channel are not ordered
                     sorted_channel_indexes = np.sort(channel_indexes)
-                    re_order = np.array([list(sorted_channel_indexes).index(ch)
-                                         for ch in channel_indexes])
+                    re_order = np.array([list(sorted_channel_indexes).index(ch) for ch in channel_indexes])
 
-            stream_mask = np.zeros(raw_unit8.shape[1], dtype='bool')
+            stream_mask = np.zeros(raw_unit8.shape[1], dtype="bool")
             for chan_ind in chan_inds:
                 chan_mask = self._mask_channels_bytes[stream_id][chan_ind]
                 stream_mask |= chan_mask
@@ -262,7 +302,7 @@ class SpikeGadgetsRawIO(BaseRawIO):
         shape = raw_unit8_mask.shape
         shape = (shape[0], shape[1] // 2)
         # reshape the and retype by view
-        raw_unit16 = raw_unit8_mask.flatten().view('int16').reshape(shape)
+        raw_unit16 = raw_unit8_mask.flatten().view("int16").reshape(shape)
 
         if re_order is not None:
             raw_unit16 = raw_unit16[:, re_order]
