@@ -18,7 +18,11 @@ from neo import AnalogSignal
 
 from neo.test.rawiotest.test_neuralynxrawio import TestNeuralynxRawIO
 
-class CommonNeuralynxIOTest(BaseTestIO, unittest.TestCase, ):
+
+class CommonNeuralynxIOTest(
+    BaseTestIO,
+    unittest.TestCase,
+):
     ioclass = NeuralynxIO
     entities_to_download = TestNeuralynxRawIO.entities_to_download
     entities_to_test = TestNeuralynxRawIO.entities_to_test
@@ -28,7 +32,7 @@ class TestCheetah_Neuraview(CommonNeuralynxIOTest, unittest.TestCase):
     files_to_test = []
 
     def test_read_block(self):
-        dirname = self.get_local_path('neuralynx/Neuraview_v2/original_data')
+        dirname = self.get_local_path("neuralynx/Neuraview_v2/original_data")
         nio = NeuralynxIO(dirname=dirname, use_cache=False)
         bl = nio.read_block()
 
@@ -37,12 +41,12 @@ class TestCheetah_Neuraview(CommonNeuralynxIOTest, unittest.TestCase):
 
 
 class TestCheetah_v551(CommonNeuralynxIOTest, unittest.TestCase):
-    cheetah_version = '5.5.1'
+    cheetah_version = "5.5.1"
     files_to_test = []
 
     def test_read_block(self):
         """Read data in a certain time range into one block"""
-        dirname = self.get_local_path('neuralynx/Cheetah_v5.5.1/original_data')
+        dirname = self.get_local_path("neuralynx/Cheetah_v5.5.1/original_data")
         nio = NeuralynxIO(dirname=dirname, use_cache=False)
 
         block = nio.read_block()
@@ -53,15 +57,14 @@ class TestCheetah_v551(CommonNeuralynxIOTest, unittest.TestCase):
         self.assertEqual(len(seg.analogsignals), 1)
         self.assertEqual(seg.analogsignals[0].shape[-1], 2)
 
-        self.assertEqual(seg.analogsignals[0].sampling_rate, 32. * pq.kHz)
+        self.assertEqual(seg.analogsignals[0].sampling_rate, 32.0 * pq.kHz)
         self.assertEqual(len(seg.spiketrains), 2)
 
         # Testing different parameter combinations
         block = nio.read_block(load_waveforms=True)
         self.assertEqual(len(block.segments[0].analogsignals), 1)
         self.assertEqual(len(block.segments[0].spiketrains), 2)
-        self.assertEqual(block.segments[0].spiketrains[0].waveforms.shape[0],
-                         block.segments[0].spiketrains[0].shape[0])
+        self.assertEqual(block.segments[0].spiketrains[0].waveforms.shape[0], block.segments[0].spiketrains[0].shape[0])
         self.assertGreater(len(block.segments[0].events), 0)
 
         # self.assertEqual(len(block.channel_indexes[-1].units[0].spiketrains), 2)  # 2 segment
@@ -73,7 +76,7 @@ class TestCheetah_v551(CommonNeuralynxIOTest, unittest.TestCase):
         # self.assertEqual(len(block.channel_indexes[-1].units), 1)  # 1 units by ChannelIndex
 
     def test_read_segment(self):
-        dirname = self.get_local_path('neuralynx/Cheetah_v5.5.1/original_data')
+        dirname = self.get_local_path("neuralynx/Cheetah_v5.5.1/original_data")
         nio = NeuralynxIO(dirname=dirname, use_cache=False)
 
         # read first segment entirely
@@ -92,12 +95,12 @@ class TestCheetah_v551(CommonNeuralynxIOTest, unittest.TestCase):
 
 
 class TestCheetah_v563(CommonNeuralynxIOTest, unittest.TestCase):
-    cheetah_version = '5.6.3'
+    cheetah_version = "5.6.3"
     files_to_test = []
 
     def test_read_block(self):
         """Read data in a certain time range into one block"""
-        dirname = self.get_local_path('neuralynx/Cheetah_v5.6.3/original_data')
+        dirname = self.get_local_path("neuralynx/Cheetah_v5.6.3/original_data")
         nio = NeuralynxIO(dirname=dirname, use_cache=False)
 
         block = nio.read_block()
@@ -107,15 +110,14 @@ class TestCheetah_v563(CommonNeuralynxIOTest, unittest.TestCase):
         for seg in block.segments:
             self.assertEqual(len(seg.analogsignals), 1)
             self.assertEqual(seg.analogsignals[0].shape[-1], 2)
-            self.assertEqual(seg.analogsignals[0].sampling_rate, 2. * pq.kHz)
+            self.assertEqual(seg.analogsignals[0].sampling_rate, 2.0 * pq.kHz)
             self.assertEqual(len(seg.spiketrains), 8)
 
         # Testing different parameter combinations
         block = nio.read_block(load_waveforms=True)
         self.assertEqual(len(block.segments[0].analogsignals), 1)
         self.assertEqual(len(block.segments[0].spiketrains), 8)
-        self.assertEqual(block.segments[0].spiketrains[0].waveforms.shape[0],
-                         block.segments[0].spiketrains[0].shape[0])
+        self.assertEqual(block.segments[0].spiketrains[0].waveforms.shape[0], block.segments[0].spiketrains[0].shape[0])
         # this is tetrode data, containing 32 samples per waveform
         self.assertEqual(block.segments[0].spiketrains[0].waveforms.shape[1], 4)
         self.assertEqual(block.segments[0].spiketrains[0].waveforms.shape[-1], 32)
@@ -130,7 +132,7 @@ class TestCheetah_v563(CommonNeuralynxIOTest, unittest.TestCase):
         # self.assertEqual(len(block.channel_indexes[-1].units), 1)  # 1 units by ChannelIndex
 
     def test_read_segment(self):
-        dirname = self.get_local_path('neuralynx/Cheetah_v5.5.1/original_data')
+        dirname = self.get_local_path("neuralynx/Cheetah_v5.5.1/original_data")
         nio = NeuralynxIO(dirname=dirname, use_cache=False)
 
         # read first segment entirely
@@ -149,11 +151,11 @@ class TestCheetah_v563(CommonNeuralynxIOTest, unittest.TestCase):
 
 
 class TestCheetah_v574(CommonNeuralynxIOTest, unittest.TestCase):
-    cheetah_version = '5.7.4'
+    cheetah_version = "5.7.4"
     files_to_test = []
 
     def test_read_block(self):
-        dirname = self.get_local_path('neuralynx/Cheetah_v5.7.4/original_data')
+        dirname = self.get_local_path("neuralynx/Cheetah_v5.7.4/original_data")
         nio = NeuralynxIO(dirname=dirname, use_cache=False)
 
         block = nio.read_block()
@@ -172,16 +174,14 @@ class TestCheetah_v574(CommonNeuralynxIOTest, unittest.TestCase):
         self.assertEqual(len(block.segments[0].spiketrains), 0)
         self.assertGreater(len(block.segments[0].events), 0)
 
-        block = nio.read_block(signal_group_mode='split-all')
+        block = nio.read_block(signal_group_mode="split-all")
         self.assertEqual(len(block.groups), 5)
 
-        block = nio.read_block(signal_group_mode='group-by-same-units')
+        block = nio.read_block(signal_group_mode="group-by-same-units")
         self.assertEqual(len(block.groups), 1)
 
     def test_read_single_file(self):
-        filename = self.get_local_path(
-            'neuralynx/Cheetah_v5.7.4/original_data/CSC1.ncs'
-        )
+        filename = self.get_local_path("neuralynx/Cheetah_v5.7.4/original_data/CSC1.ncs")
         nio = NeuralynxIO(filename=filename, use_cache=False)
         block = nio.read_block()
         self.assertTrue(len(block.segments[0].analogsignals) > 0)
@@ -190,12 +190,10 @@ class TestCheetah_v574(CommonNeuralynxIOTest, unittest.TestCase):
         self.assertTrue((len(block.segments[0].epochs)) == 0)
 
     def test_exclude_filename(self):
-        dname = self.get_local_path(
-            'neuralynx/Cheetah_v5.7.4/original_data/'
-        )
+        dname = self.get_local_path("neuralynx/Cheetah_v5.7.4/original_data/")
 
         # exclude a single file
-        nio = NeuralynxIO(dirname=dname, exclude_filename='CSC1.ncs', use_cache=False)
+        nio = NeuralynxIO(dirname=dname, exclude_filename="CSC1.ncs", use_cache=False)
         block = nio.read_block()
         self.assertTrue(len(block.segments[0].analogsignals) > 0)
         self.assertTrue((len(block.segments[0].spiketrains)) >= 0)
@@ -203,9 +201,8 @@ class TestCheetah_v574(CommonNeuralynxIOTest, unittest.TestCase):
         self.assertTrue((len(block.segments[0].epochs)) == 0)
 
         # exclude all ncs files from session
-        exclude_files = [f'CSC{i}.ncs' for i in range(6)]
-        nio = NeuralynxIO(dirname=dname, exclude_filename=exclude_files,
-                          use_cache=False)
+        exclude_files = [f"CSC{i}.ncs" for i in range(6)]
+        nio = NeuralynxIO(dirname=dname, exclude_filename=exclude_files, use_cache=False)
         block = nio.read_block()
         self.assertTrue(len(block.segments[0].analogsignals) == 0)
         self.assertTrue((len(block.segments[0].spiketrains)) >= 0)
@@ -214,11 +211,11 @@ class TestCheetah_v574(CommonNeuralynxIOTest, unittest.TestCase):
 
 
 class TestPegasus_v211(CommonNeuralynxIOTest, unittest.TestCase):
-    pegasus_version = '2.1.1'
+    pegasus_version = "2.1.1"
     files_to_test = []
 
     def test_read_block(self):
-        dirname = self.get_local_path('neuralynx/Pegasus_v2.1.1')
+        dirname = self.get_local_path("neuralynx/Pegasus_v2.1.1")
         nio = NeuralynxIO(dirname=dirname, use_cache=False)
 
         block = nio.read_block()
@@ -234,9 +231,9 @@ class TestPegasus_v211(CommonNeuralynxIOTest, unittest.TestCase):
         self.assertEqual(len(block.segments[0].spiketrains), 0)
         self.assertGreater(len(block.segments[0].events), 1)
 
-        block = nio.read_block(signal_group_mode='split-all')
+        block = nio.read_block(signal_group_mode="split-all")
 
-        block = nio.read_block(signal_group_mode='group-by-same-units')
+        block = nio.read_block(signal_group_mode="group-by-same-units")
 
 
 class TestData(CommonNeuralynxIOTest, unittest.TestCase):
@@ -255,12 +252,12 @@ class TestData(CommonNeuralynxIOTest, unittest.TestCase):
                 vals = list(map(int, line.split()))
                 numSampsThisLine = len(vals) - 4
                 if numSampsThisLine < 0 or numSampsThisLine < vals[3]:
-                    raise IOError('plain data file "' + filename + ' improperly formatted')
+                    raise IOError('plain data file "' + filename + " improperly formatted")
                 numAvail = min(numSampsThisLine, vals[3])  # only use valid samples
                 if numAvail < numSamps - len(res):
-                    res.append(vals[4:(4 + numAvail)])
+                    res.append(vals[4 : (4 + numAvail)])
                 else:
-                    res.append(vals[4:(4 + numSamps - len(res))])
+                    res.append(vals[4 : (4 + numSamps - len(res))])
                 totRes += len(res[-1])
                 if totRes == numSamps:
                     break
@@ -275,22 +272,25 @@ class TestData(CommonNeuralynxIOTest, unittest.TestCase):
 
             # check that data agrees in first segment first channel only
             for anasig_id, anasig in enumerate(block.segments[0].analogsignals):
-                chid = anasig.array_annotations['channel_ids'][0]
+                chid = anasig.array_annotations["channel_ids"][0]
 
-                chname = str(anasig.array_annotations['channel_names'][0])
+                chname = str(anasig.array_annotations["channel_names"][0])
                 chuid = (chname, chid)
-                filename = nio.ncs_filenames[chuid][:-3] + 'txt'
-                filename = filename.replace('original_data', 'plain_data')
+                filename = nio.ncs_filenames[chuid][:-3] + "txt"
+                filename = filename.replace("original_data", "plain_data")
                 overlap = 512 * 500
                 if os.path.isfile(filename):
                     plain_data = self._load_plaindata(filename, overlap)
                     gain_factor_0 = plain_data[0] / anasig.magnitude[0, 0]
                     numToTest = min(len(plain_data), len(anasig.magnitude[:, 0]))
-                    np.testing.assert_allclose(plain_data[:numToTest],
-                                               anasig.magnitude[:numToTest, 0] * gain_factor_0,
-                                               rtol=0.01, err_msg=" for file " + filename)
+                    np.testing.assert_allclose(
+                        plain_data[:numToTest],
+                        anasig.magnitude[:numToTest, 0] * gain_factor_0,
+                        rtol=0.01,
+                        err_msg=" for file " + filename,
+                    )
                 else:
-                    warnings.warn(f'Could not find corresponding test file {filename}')
+                    warnings.warn(f"Could not find corresponding test file {filename}")
                     # TODO: Create missing plain data file using NeuraView
                     # https://neuralynx.com/software/category/data-analysis
 
@@ -301,13 +301,13 @@ class TestData(CommonNeuralynxIOTest, unittest.TestCase):
             block = nio.read_block()
 
             for st in block.segments[0].spiketrains:
-                filename = st.file_origin.replace('original_data', 'plain_data')
-                if '.nse' in st.file_origin:
-                    filename = filename.replace('.nse', '.txt')
+                filename = st.file_origin.replace("original_data", "plain_data")
+                if ".nse" in st.file_origin:
+                    filename = filename.replace(".nse", ".txt")
                     times_column = 0
                     plain_data = np.loadtxt(filename)[:, times_column]
-                elif '.ntt' in st.file_origin:
-                    filename = filename.replace('.ntt', '.txt')
+                elif ".ntt" in st.file_origin:
+                    filename = filename.replace(".ntt", ".txt")
                     times_column = 2
                     plain_data = np.loadtxt(filename)[:, times_column]
                     # ntt files contain 4 rows per spike time
@@ -320,7 +320,7 @@ class TestData(CommonNeuralynxIOTest, unittest.TestCase):
 
 class TestIncompleteBlocks(CommonNeuralynxIOTest, unittest.TestCase):
     def test_incomplete_block_handling_v632(self):
-        dirname = self.get_local_path('neuralynx/Cheetah_v6.3.2/incomplete_blocks')
+        dirname = self.get_local_path("neuralynx/Cheetah_v6.3.2/incomplete_blocks")
         nio = NeuralynxIO(dirname=dirname, use_cache=False)
 
         block = nio.read_block()
@@ -344,7 +344,7 @@ class TestIncompleteBlocks(CommonNeuralynxIOTest, unittest.TestCase):
 
 class TestGaps(CommonNeuralynxIOTest, unittest.TestCase):
     def test_gap_handling_v551(self):
-        dirname = self.get_local_path('neuralynx/Cheetah_v5.5.1/original_data')
+        dirname = self.get_local_path("neuralynx/Cheetah_v5.5.1/original_data")
         nio = NeuralynxIO(dirname=dirname, use_cache=False)
 
         block = nio.read_block()
@@ -357,7 +357,7 @@ class TestGaps(CommonNeuralynxIOTest, unittest.TestCase):
         # self.assertEqual(len(block.channel_indexes[-1].units[0].spiketrains), n_gaps + 1)
 
     def test_gap_handling_v563(self):
-        dirname = self.get_local_path('neuralynx/Cheetah_v5.6.3/original_data')
+        dirname = self.get_local_path("neuralynx/Cheetah_v5.6.3/original_data")
         nio = NeuralynxIO(dirname=dirname, use_cache=False)
         block = nio.read_block()
 
@@ -372,7 +372,7 @@ class TestGaps(CommonNeuralynxIOTest, unittest.TestCase):
 class TestMultiSamplingRates(CommonNeuralynxIOTest, unittest.TestCase):
     def test_multi_sampling_rates(self):
         # Test Cheetah 6.4.1, with different sampling rates across ncs files.
-        nio = NeuralynxIO(self.get_local_path('neuralynx/Cheetah_v6.4.1dev/original_data'))
+        nio = NeuralynxIO(self.get_local_path("neuralynx/Cheetah_v6.4.1dev/original_data"))
         block = nio.read_block()
 
         self.assertEqual(len(block.segments), 1)
@@ -385,23 +385,23 @@ class TestMultiSamplingRates(CommonNeuralynxIOTest, unittest.TestCase):
         for sig in block.segments[0].analogsignals:
             observed_rate = sig.sampling_rate.rescale(expected_rates.units)
             # using isclose for flexible float comparison
-            self.assertTrue(any([np.isclose(observed_rate, exp_rate)]) for
-                            exp_rate in expected_rates)
+            self.assertTrue(any([np.isclose(observed_rate, exp_rate)]) for exp_rate in expected_rates)
 
 
 def compare_neo_content(bl1, bl2):
-    print('*' * 5, 'Comparison of blocks', '*' * 5)
-    object_types_to_test = [Segment, AnalogSignal,
-                            SpikeTrain, Event, Epoch]
+    print("*" * 5, "Comparison of blocks", "*" * 5)
+    object_types_to_test = [Segment, AnalogSignal, SpikeTrain, Event, Epoch]
     for objtype in object_types_to_test:
-        print('Testing {}'.format(objtype))
+        print("Testing {}".format(objtype))
         children1 = bl1.list_children_by_class(objtype)
         children2 = bl2.list_children_by_class(objtype)
 
         if len(children1) != len(children2):
-            warnings.warn('Number of {} is different in both blocks ({} != {}).'
-                          ' Skipping comparison'
-                          ''.format(objtype, len(children1), len(children2)))
+            warnings.warn(
+                "Number of {} is different in both blocks ({} != {})."
+                " Skipping comparison"
+                "".format(objtype, len(children1), len(children2))
+            )
             continue
 
         for child1, child2 in zip(children1, children2):
@@ -411,8 +411,10 @@ def compare_neo_content(bl1, bl2):
 
 def compare_annotations(anno1, anno2):
     if len(anno1) != len(anno2):
-        warnings.warn('Different numbers of annotations! {} != {}\nSkipping further comparison of '
-                      'this annotation list.'.format(anno1.keys(), anno2.keys()))
+        warnings.warn(
+            "Different numbers of annotations! {} != {}\nSkipping further comparison of "
+            "this annotation list.".format(anno1.keys(), anno2.keys())
+        )
         return
     assert anno1.keys() == anno2.keys()
     for key in anno1.keys():
@@ -424,22 +426,28 @@ def compare_attributes(child1, child2):
     for attr_id in range(len(child1._all_attrs)):
         attr_name = child1._all_attrs[attr_id][0]
         attr_dtype = child1._all_attrs[attr_id][1]
-        if type(child1) == AnalogSignal and attr_name == 'signal':
+        if type(child1) == AnalogSignal and attr_name == "signal":
             continue
-        if type(child1) == SpikeTrain and attr_name == 'times':
+        if type(child1) == SpikeTrain and attr_name == "times":
             continue
         unequal = child1.__getattribute__(attr_name) != child2.__getattribute__(attr_name)
 
-        if hasattr(unequal, 'any'):
+        if hasattr(unequal, "any"):
             unequal = unequal.any()
         if unequal:
-            warnings.warn('Attributes differ! {}.{}={} is not equal to {}.{}={}'
-                          ''.format(child1.__class__.__name__, attr_name,
-                                    child1.__getattribute__(attr_name),
-                                    child2.__class__.__name__, attr_name,
-                                    child2.__getattribute__(attr_name)))
+            warnings.warn(
+                "Attributes differ! {}.{}={} is not equal to {}.{}={}"
+                "".format(
+                    child1.__class__.__name__,
+                    attr_name,
+                    child1.__getattribute__(attr_name),
+                    child2.__class__.__name__,
+                    attr_name,
+                    child2.__getattribute__(attr_name),
+                )
+            )
 
 
-if __name__ == '__main__':
+if __name__ == "__main__":
     unittest.main()
     # ~ compare_old_and_new_neuralynxio()
