@@ -8,15 +8,16 @@ from neo.io import PlexonIO
 from neo.test.iotest.common_io_test import BaseTestIO
 
 
-class TestPlexonIO(BaseTestIO, unittest.TestCase, ):
+class TestPlexonIO(
+    BaseTestIO,
+    unittest.TestCase,
+):
     ioclass = PlexonIO
-    entities_to_download = [
-        'plexon'
-    ]
+    entities_to_download = ["plexon"]
     entities_to_test = [
-        'plexon/File_plexon_1.plx',
-        'plexon/File_plexon_2.plx',
-        'plexon/File_plexon_3.plx',
+        "plexon/File_plexon_1.plx",
+        "plexon/File_plexon_2.plx",
+        "plexon/File_plexon_3.plx",
     ]
 
 
